using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.Ink;

namespace InkAnalysisSampleApp
{
    public partial class MainForm : Form
    {
        private InkCollector inkCollector;


        public MainForm()
        {
            InitializeComponent();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            inkCollector = new InkCollector(grpInkArea.Handle);
            inkCollector.Enabled = true;

        }

        private void btnShowBaseline_Click(object sender, EventArgs e)
        {
            // If there are no Strokes, there is no Ink
            if (inkCollector.Ink.Strokes.Count == 0)
            {
                return;
            }

            RecognizerContext recoContext = new RecognizerContext();
            recoContext.Strokes = this.inkCollector.Ink.Strokes;

            RecognitionStatus status = RecognitionStatus.NoError;
            RecognitionResult result = recoContext.Recognize(out status);

            if (status == RecognitionStatus.NoError)
            {
                Line baseLine = result.TopAlternate.Baseline;
                Graphics g = grpInkArea.CreateGraphics();
                Renderer renderer = inkCollector.Renderer;

                Point beginPoint = baseLine.BeginPoint;
                Point endPoint = baseLine.EndPoint;
 
                renderer.InkSpaceToPixel(g, ref beginPoint);
                renderer.InkSpaceToPixel(g, ref endPoint);

                g.DrawLine(Pens.Blue, beginPoint, endPoint);
            }
            recoContext.Dispose();

        }

        private void btnClearInkArea_Click(object sender, EventArgs e)
        {
            this.inkCollector.Ink.DeleteStrokes();
            this.grpInkArea.Invalidate();
        }

        private void btnExit_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void btnRecognizeText_Click(object sender, EventArgs e)
        {
            // Old Way
            string results = inkCollector.Ink.Strokes.ToString();
            MessageBox.Show(results);
        }

        private void btnRecognizeTextAsEmail_Click(object sender, EventArgs e)
        {

            InkAnalyzer inkAnalyzer = new InkAnalyzer(this.inkCollector.Ink, this);
            inkAnalyzer.AddStrokes(this.inkCollector.Ink.Strokes);


            AnalysisHintNode hintNode = inkAnalyzer.CreateAnalysisHint();
            hintNode.Location.MakeInfinite();

            hintNode.Name = Factoid.Email;

            inkAnalyzer.Analyze();

            MessageBox.Show(inkAnalyzer.GetRecognizedString());


        }

        private void btnRecognizeTextAsUrl_Click(object sender, EventArgs e)
        {

            InkAnalyzer inkAnalyzer = new InkAnalyzer(this.inkCollector.Ink, this);
            inkAnalyzer.AddStrokes(this.inkCollector.Ink.Strokes);

            AnalysisHintNode hintNode = inkAnalyzer.CreateAnalysisHint();
            hintNode.Location.MakeInfinite();

            hintNode.Name = Factoid.Date;

            inkAnalyzer.Analyze();

            MessageBox.Show(inkAnalyzer.GetRecognizedString());

        }

        private void btnRecognizeWordFromList_Click(object sender, EventArgs e)
        {

            InkAnalyzer inkAnalyzer = new InkAnalyzer(this.inkCollector.Ink, this);
            inkAnalyzer.AddStrokes(this.inkCollector.Ink.Strokes);

            string[] wordListArray = { "cereberal", "hyperbaric", "wound", "anemic" };

            AnalysisHintNode hintNode = inkAnalyzer.CreateAnalysisHint();
            hintNode.Location.MakeInfinite();
            hintNode.SetWordlist(wordListArray);

            hintNode.Name = Factoid.WordList;

            // Same as the line above
            //hintNode.Name = "Wordlist";

            inkAnalyzer.Analyze();

            MessageBox.Show(inkAnalyzer.GetRecognizedString());

        }


        private void btnShowAlternates_Click(object sender, EventArgs e)
        {
            InkAnalyzer inkAnalyzer = new InkAnalyzer(this.inkCollector.Ink, this);
            inkAnalyzer.AddStrokes(this.inkCollector.Ink.Strokes);
            inkAnalyzer.Analyze();

            string msgText = "Matches & Confidence Levels:\n------------------------------------\n";
            foreach (AnalysisAlternate recoAlternates in inkAnalyzer.GetAlternates())
            {
                msgText += recoAlternates.RecognizedString + " (" + recoAlternates.InkRecognitionConfidence.ToString() + ")\n";
            }

            MessageBox.Show(msgText);
        }

        private void btnAnalyzeStructure_Click(object sender, EventArgs e)
        {
            InkAnalyzer inkAnalyzer = new InkAnalyzer(this.inkCollector.Ink, this.grpInkArea);
            
            // Remove event to avoid a storm of MessageBoxes
            inkAnalyzer.ContextNodeCreated -= new ContextNodeCreatedEventHandler(inkAnalyzer_ContextNodeCreated);

            inkAnalyzer.AddStrokes(this.inkCollector.Ink.Strokes);
            inkAnalyzer.Analyze();
            
            StringBuilder sbNodes = new StringBuilder();

            sbNodes.Append("Structure Analysis and Recognition Results\n");
            sbNodes.Append("------------------------------------------\n");

            this.TraverseNodes(inkAnalyzer.RootNode, sbNodes, 0);
            MessageBox.Show(sbNodes.ToString());

        }


        private void TraverseNodes(ContextNode contextNode, StringBuilder nodes, int Depth)
        {
    
            nodes.Append(string.Empty.PadLeft(Depth * 4, ' '));
            nodes.Append(contextNode.GetType().ToString() + "  ");
            nodes.Append("[" + contextNode.Strokes.ToString() + "]");
            nodes.Append("\n");
            foreach (ContextNode subNode in contextNode.SubNodes)
            {
                this.TraverseNodes(subNode, nodes, Depth + 1);
            }
        }

        private void btnDetectGestures_Click(object sender, EventArgs e)
        {
            //inkCollector.CollectionMode = CollectionMode.InkAndGesture;

            inkCollector.CollectionMode = CollectionMode.GestureOnly;

            inkCollector.SetGestureStatus(ApplicationGesture.Circle, true);

            this.inkCollector.Gesture += new InkCollectorGestureEventHandler(inkCollector_Gesture);
        }

        void inkCollector_Gesture(object sender, InkCollectorGestureEventArgs e)
        {
            if (e.Gestures[0].Id == ApplicationGesture.Circle)
            {
                if (e.Gestures[0].Confidence == RecognitionConfidence.Strong)
                {
                    MessageBox.Show("I am very certain you drew a circle Gesture.");
                }
                if (e.Gestures[0].Confidence == RecognitionConfidence.Intermediate)
                {
                    MessageBox.Show("I am reasonably certain you drew a circle Gesture.");
                }
            }
        }

        private void btnAnalyzeDrawings_Click(object sender, EventArgs e)
        {
            InkAnalyzer inkAnalyzer = new Microsoft.Ink.InkAnalyzer(inkCollector.Ink, this);
            inkAnalyzer.AddStrokes(inkCollector.Ink.Strokes);
            inkAnalyzer.Analyze();

            ContextNodeCollection drawingNodes = inkAnalyzer.FindNodesOfType(ContextNodeType.InkDrawing);
            string results = "Drawings detected:\n";
            foreach (InkDrawingNode drawingNode in drawingNodes)
            {
                results += drawingNode.GetShapeName() + "\n";
            }
            MessageBox.Show(results);

        }

        private void FindLinkedNodes(ContextNode contextNode)
        {
            if (contextNode.Links.Count > 0)
            {
                foreach (ContextLink contextLink in contextNode.Links)
                {
                    
                    contextLink.SourceNode.Strokes.ModifyDrawingAttributes(new DrawingAttributes(Color.Blue));
                    contextLink.DestinationNode.Strokes.ModifyDrawingAttributes(new DrawingAttributes(Color.Red));
                }
            }

            // Recursively Parse Tree
            foreach (ContextNode subNode in contextNode.SubNodes)
            {
                FindLinkedNodes(subNode);
            }
        }

        private void btnShowLinks_Click(object sender, EventArgs e)
        {
            InkAnalyzer inkAnalyzer = new InkAnalyzer(inkCollector.Ink, this);
            inkAnalyzer.AddStrokes(inkCollector.Ink.Strokes);
            inkAnalyzer.Analyze();
            FindLinkedNodes(inkAnalyzer.RootNode);
            
            // Force Redraw
            Refresh();

        }

        private void btnEvents_Click(object sender, EventArgs e)
        {
            InkAnalyzer inkAnalyzer = new InkAnalyzer(this.inkCollector.Ink, this.grpInkArea);

            inkAnalyzer.ContextNodeCreated += new ContextNodeCreatedEventHandler(inkAnalyzer_ContextNodeCreated);
            inkAnalyzer.ContextNodeLinkAdding += new ContextNodeLinkAddingEventHandler(inkAnalyzer_ContextNodeLinkAdding);

            inkAnalyzer.AddStrokes(this.inkCollector.Ink.Strokes);
            inkAnalyzer.Analyze();

        }

        void inkAnalyzer_ContextNodeLinkAdding(object sender, ContextNodeLinkAddingEventArgs e)
        {
            MessageBox.Show("Link added to " + e.LinkToBeAdded.DestinationNode.Strokes.RecognitionResult + ".");
        }

        void inkAnalyzer_ContextNodeCreated(object sender, ContextNodeCreatedEventArgs e)
        {
            MessageBox.Show("Node Created.");
        }


    }
}